#include <string.h>

#include "outstreams.hpp"
#include "utility.hpp"


void removeChars( const char* chars, string* s )
{
  string::size_type i = s->find_first_of( chars );

  while( i != string::npos )
  {
    s->erase( i, 1 );
    i = s->find_first_of( chars, i );
  }
}


void replaceChars( const char* chars, const char* with, string* s )
{
  string::size_type i = s->find_first_of( chars );

  while( i != string::npos )
  {
    s->erase( i, 1 );

    s->insert( i, with );

    i = s->find_first_of( chars, i + strlen( with ) );
  }
}


void makeLower( string* s )
{
  string lowered;

  for( int i = 0; i < s->size(); ++i )
  {
    char c = s->at( i );

    if( isupper( c ) )
    {
      lowered.append( 1, tolower( c ) );
    } else {
      lowered.append( 1, c );
    }
  }

  s->assign( lowered );
}


void trimLeadingSpaces( string* s )
{
  while( s->length() > 0 && s->at( 0 ) == ' ' )
  {
    s->erase( 0, 1 );
  }
}


void trimTrailingSpaces( string* s )
{
  while( s->length() > 0 && s->at( s->length() - 1 ) == ' ' )
  {
    s->erase( s->length() - 1, 1 );
  }
}


float xtof_helper( char c )
{
  if( isdigit( c ) )
  {
    return (float) (c-48);
  }

  switch( c )
  {
    case 'a': case 'A': return 10.0f;
    case 'b': case 'B': return 11.0f;
    case 'c': case 'C': return 12.0f;
    case 'd': case 'D': return 13.0f;
    case 'e': case 'E': return 14.0f;
    case 'f': case 'F': return 15.0f;
  }
}

float xtof( char c1, char c0 )
{
  if( !isxdigit( c1 ) ) { printError( "'%c' is not a hex digit.\n", c1 ); exit( -1 ); }
  if( !isxdigit( c0 ) ) { printError( "'%c' is not a hex digit.\n", c0 ); exit( -1 ); }

  float f = 16.0f*xtof_helper( c1 ) + xtof_helper( c0 );

  return f / 255.0f;
}


void formatPath( string* s )
{
  // paths should have
  // replace all forward slashes with backslashes to
  // ensure consistent path characters
  size_t lookHere = 0;
  size_t foundHere;
  while( (foundHere = s->find( "/", lookHere ))
         != string::npos )
  {
    s->replace( foundHere, 1, "\\" );
    lookHere = foundHere + 1;
  }

  // if the filename ends in a "\" we will interpret
  // it as a directory, but the trailing slash wigs out the
  // call to stat so trim any here
  while( s->size() > 0 &&
         s->at( s->size() - 1 ) == '\\' )
  {
    s->erase( s->size() - 1 );
  }
}
